package jp.starlogic.jms.saler;

import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.StreamMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.naming.NamingException;

import jp.starlogic.jms.axis.OrderCounter;
import jp.starlogic.jms.axis.OrderInfo;
import jp.starlogic.jms.jndi.JmsConnectionFactory;

public class AgentImpl implements Agent, MessageListener {
	private JmsConnectionFactory factory;

	private TopicConnection tConnect;
	private TopicSession tSession;
	private TopicPublisher tPublisher;

	private QueueConnection qConnect;
	private QueueSession qSession;
	private QueueReceiver qReceiver;
	private Queue receiveQueue;

	private Topic hotDealTopic;
	
	private long lifeTime ; 

	private OrderCounter counter;


	public void setFactory(JmsConnectionFactory factory) {
		this.factory = factory;
	}

	public void init(OrderCounter counter, String username, String password, long lifeTime)
			throws NamingException, JMSException {
		this.counter = counter;
		this.lifeTime = lifeTime;
		
		
		TopicConnectionFactory tFactory = factory.getTopicConnectionFactory();
		QueueConnectionFactory qFactory = factory.getQueueConnectionFactory();

		tConnect = tFactory.createTopicConnection(username, password);
		qConnect = qFactory.createQueueConnection(username, password);

		tSession = tConnect.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
		qSession = qConnect.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

		hotDealTopic = tSession.createTopic("Hot Deals");
		receiveQueue = qSession.createQueue("Receive Queue");

		tPublisher = tSession.createPublisher(hotDealTopic);

		qReceiver = qSession.createReceiver(receiveQueue);
		qReceiver.setMessageListener(this);

		tConnect.start();
		qConnect.start();

	}

	public void onMessage(Message message) {
		try {
			StreamMessage strmMsg = (StreamMessage) message;

			OrderInfo info = new OrderInfo();
			info.setOrderNo(strmMsg.readInt());
			info.setKind(strmMsg.readString());
			info.setItem(strmMsg.readString());
			info.setQuantity(strmMsg.readInt());
			
			System.out.println("HOGE");
			counter.orderComplete(info);

		} catch (JMSException e) {
			e.printStackTrace();
		}
	}

	public void order(OrderInfo info) {
		try {
			StreamMessage message = tSession.createStreamMessage();
			message.writeInt(info.getOrderNo());
			message.writeString(info.getKind());
			message.writeString(info.getItem());
			message.writeInt(info.getQuantity());

			// message.setStringProperty("Username", username);
			// message.setStringProperty("Itemdesc", itemDesc);
			message.setStringProperty("target", info.getKind());

			message.setJMSReplyTo(receiveQueue);

			tPublisher.publish(message, DeliveryMode.PERSISTENT,
					Message.DEFAULT_PRIORITY, lifeTime);
		} catch (JMSException e) {
			e.printStackTrace();
		}
	}

}
