package jp.starlogic.jms.saler;

import java.io.BufferedReader;
import java.io.InputStreamReader;

import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.StreamMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.naming.NamingException;

import jp.starlogic.jms.jndi.JmsConnectionFactory;

import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.factory.S2ContainerFactory;

public class Saler implements MessageListener {
	private static final long LIFE_TIME = 1800000;
	
	private static final String PATH = "saler.dicon";
	private S2Container container;

	// wǖ
	private static final String SUBSCRIBE_NAME = "Saler Subscribe";

	// ڑIDʗp
	private static final String TCONN_MK = "_T";
//	private static final String QCONN_MK = "_Q";

	// x(msec)
	private long waitingTime = 0;

	private QueueConnection qConnect;
	private QueueSession qSession;
	private QueueSender qSender;

	private TopicConnection tConnect;
	private TopicSession tSession;

	private TopicSubscriber tSubscriber;
	private Topic hotDealsTopic;
	private static String uname;

	public Saler(String username, String password, String salerKind,
			long waitingTime) throws NamingException, JMSException {
		uname = username;
		this.waitingTime = waitingTime;

		containerInit();
		miscInit(username, password, salerKind);
	}

	private void containerInit() {
		container = S2ContainerFactory.create(PATH);
		container.init();
	}

	private static final String SELECTOR_STR = "target=";

	private void miscInit(String username, String password, String salerKind)
			throws NamingException, JMSException {
		JmsConnectionFactory factory = (JmsConnectionFactory) container
				.getComponent("jmsConnectionFactory");

		TopicConnectionFactory tFactory = factory.getTopicConnectionFactory();
		QueueConnectionFactory qFactory = factory.getQueueConnectionFactory();

		tConnect = tFactory.createTopicConnection(username, password);
		qConnect = qFactory.createQueueConnection(username, password);
		tConnect.setClientID(username + TCONN_MK);
//		qConnect.setClientID(username + QCONN_MK);
		tSession = tConnect.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
		qSession = qConnect.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

		hotDealsTopic = tSession.createTopic("Hot Deals");

		tSubscriber = tSession.createDurableSubscriber(hotDealsTopic,
				SUBSCRIBE_NAME, getSelector(salerKind), false);

		tSubscriber.setMessageListener(this);
		tConnect.start();
	}
	
	private String getSelector(String sel) {
		return SELECTOR_STR + "'" + sel + "'";
	}

	private void sell(Message msg) {
		try {
			StreamMessage strmMsg = (StreamMessage) msg;
			int orderNo = strmMsg.readInt();
			String kind = strmMsg.readString();
			String item = strmMsg.readString();
			int quantity = strmMsg.readInt();

			System.out.println("Received order :" + orderNo + " " + item + "  "
					+ quantity);
			
			//	w莞ԃfBC
			Thread.sleep(waitingTime);

			StreamMessage resMsg = qSession.createStreamMessage();
			resMsg.writeInt(orderNo);
			resMsg.writeString(kind);
			resMsg.writeString(item);
			resMsg.writeInt(quantity);
			resMsg.setJMSCorrelationID(uname);

			Queue buyQueue = (Queue) strmMsg.getJMSReplyTo();
			qSender = qSession.createSender(buyQueue);
			qSender.send(resMsg, DeliveryMode.PERSISTENT,
					Message.DEFAULT_PRIORITY, LIFE_TIME);
			
			System.out.println("Sent response");
		} catch (JMSException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
		}

	}

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		String username, password, salerKind;
		long waitingTime;

		if (args.length == 4) {
			username = args[0];
			password = args[1];
			salerKind = args[2];
			String tmp = args[3];
			waitingTime = Long.valueOf(tmp).longValue();
		} else {
			System.out.println("s : ");
			System.out
					.println("Java Saler username password salerKind waitingTime");
			return;
		}

		Saler saler = new Saler(username, password, salerKind, waitingTime);

		System.out
				.println("\nSaler application started as " + salerKind + "\n");

		BufferedReader stdin = new BufferedReader(new InputStreamReader(
				System.in));

		while (true) {
			String s = stdin.readLine();
			if (s == null || s.equals("")) {
				saler.exit();
			} else if (s.equalsIgnoreCase("unsubscribe")) {
				saler.exitWithUnsubscribe();
			} else if (s.equals("commit")) {
				saler.commit();
			}
		}
	}

	private void exit() {
		try {
			tSubscriber.close();
		} catch (JMSException e) {
			e.printStackTrace();
		}
		System.exit(0);
	}

	private void exitWithUnsubscribe() {
		try {
			tSubscriber.close();
			tSession.unsubscribe(SUBSCRIBE_NAME);
		} catch (JMSException e) {
			e.printStackTrace();
		}
		System.exit(0);
	}
	
	private void commit() {
		try {
			tSession.commit();
		} catch (JMSException e) {
			// TODO ꂽ catch ubN
			e.printStackTrace();
		}
	}

	public void onMessage(Message msg) {
//		try {
//			tSession.commit();
			System.out.println("HOGEHOGE");
			sell(msg);
//		} catch (JMSException e) {
//			// TODO ꂽ catch ubN
//			e.printStackTrace();
//		}
	}

}
